/***************************************************************************
 *
 * Copyright (c) 2000-2001 BalaBit IT Ltd, Budapest, Hungary
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: cap.c,v 1.2.2.1 2001/02/22 14:06:27 bazsi Exp $
 *
 ***************************************************************************/

#include <zorp/cap.h>
#include <zorp/log.h>


#if ENABLE_CAPS

#define CORE_ERROR "core.error"
#define CORE_CAPS "core.caps"

gchar *zorp_caps = "cap_net_admin,cap_net_bind_service=p";

gboolean 
cap_modify(int capability, int onoff)
{
  cap_t caps;
  gchar *cap_txt;

  if (!zorp_caps)
    return TRUE;
  
  caps = cap_get_proc();
  if (!caps)
    return FALSE;


  if (cap_set_flag(caps, CAP_EFFECTIVE, 1, &capability, onoff) == -1)
    {
      cap_free(caps);
      return FALSE;
    }

  cap_txt = cap_to_text(caps, NULL);
  z_log(NULL, CORE_CAPS, 5, "Setting capabilities: %s", cap_txt);
  if (cap_set_proc(caps) == -1)
    {
      z_log(NULL, CORE_ERROR, 3, "Error setting capabilities: %s", cap_txt);
      cap_free(cap_txt);
      cap_free(caps);
      return FALSE;
    }
  cap_free(cap_txt);
  cap_free(caps);
  return TRUE;
}

cap_t 
cap_save(void)
{
  if (!zorp_caps)
    return NULL;
  return cap_get_proc();
}

gboolean
cap_restore(cap_t r)
{
  gboolean rc;
  gchar *cap_txt;

  if (!zorp_caps)
    return TRUE;
  cap_txt = cap_to_text(r, NULL);
  z_log(NULL, CORE_CAPS, 5, "Resetting capabilities: %s", cap_txt);
  rc = cap_set_proc(r) != -1;
  if (!rc)
    {
      z_log(NULL, CORE_CAPS, 5, "Error setting capabilities: %s", cap_txt);
    }
  cap_free(r);
  cap_free(cap_txt);
  return rc;
}

#endif
