/***************************************************************************
 *
 * Copyright (c) 2000 BalaBit IT Ltd, Budapest, Hungary
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: freeq.c,v 1.3 2000/12/20 16:28:30 sasa Exp $
 *
 * Author  : Bazsi
 * Auditor : 
 * Last version: 
 * Notes:
 *
 ***************************************************************************/

#include <zorp/freeq.h>
#include <zorp/log.h>

#define FREE_QUEUE_SIZE 64
#define FREE_QUEUE_NEXT_SLOT(p) ((p < FREE_QUEUE_SIZE-1) ? p + 1 : 0)

static ZFreeQueue *freeq;

typedef struct _ZFreeQueueItem
{
  gpointer f;
  ZFreeFunc freefn;
} ZFreeQueueItem;

typedef struct _ZRealFreeQueue 
{
  GMutex *lock;
  GCond *queue_flushed;
  ZFreeQueueItem free_queue[FREE_QUEUE_SIZE];
  gint first_free;
} ZRealFreeQueue;

static ZFreeQueue *
z_free_queue_new(void)
{
  ZRealFreeQueue *self = g_new0(ZRealFreeQueue, 1);
  
  self->lock = g_mutex_new();
  self->queue_flushed = g_cond_new();
  return (ZFreeQueue *) self;
}

static void
z_free_queue_free(ZFreeQueue *s)
{
  ZRealFreeQueue *self = (ZRealFreeQueue *) s;
  
  g_mutex_free(self->lock);
  g_cond_free(self->queue_flushed);
  g_free(self);
}

static void
z_free_queue_add_item(ZFreeQueue *s, gpointer f, ZFreeFunc freefn)
{
  ZRealFreeQueue *self = (ZRealFreeQueue *) s;
  
  g_mutex_lock(self->lock);
  if (self->first_free >  FREE_QUEUE_SIZE)
    {
      z_log(NULL, "core.debug", 5, "Free queue full, for a performance tuning increase FREE_QUEUE_SIZE.");
      g_cond_wait(self->queue_flushed, self->lock);
    }
  self->free_queue[self->first_free].f = f;
  self->free_queue[self->first_free].freefn = freefn;
  self->first_free++;
  g_mutex_unlock(self->lock);
}

static void
z_free_queue_flush_items(ZFreeQueue *s)
{
  ZRealFreeQueue *self = (ZRealFreeQueue *) s;
  gint i;

  g_mutex_lock(self->lock);
  for (i = 0; i < self->first_free; i++)
    {
      if (self->free_queue[i].freefn)
        self->free_queue[i].freefn(self->free_queue[i].f);
      else
        g_free(self->free_queue[i].f);
    }
  self->first_free = 0;
  g_mutex_unlock(self->lock);
  g_cond_signal(self->queue_flushed);
}

void
z_free_queue_init(void)
{
  freeq = z_free_queue_new();
}

void
z_free_queue_destroy(void)
{
  z_free_queue_free(freeq);
  freeq = NULL;
}

void
z_free_queue_add(gpointer f, ZFreeFunc freefn)
{
  z_free_queue_add_item(freeq, f, freefn);
}

void
z_free_queue_flush(void)
{
  z_free_queue_flush_items(freeq);
}
