/***************************************************************************
 *
 * Copyright (c) 2000 BalaBit IT Ltd, Budapest, Hungary
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: ipchains.c,v 1.1 2001/01/02 11:34:42 bazsi Exp $
 *
 ***************************************************************************/

#include <zorp/ipchains.h>
#include <zorp/log.h>
#include <zorp/cap.h>

#include <string.h>
#include <unistd.h>

#define CORE_ERROR "core.error"

/* ipchains specific */

struct _ZIPChains 
{
  gint fd;
};

ZIPChains *
z_ipchains_new(void)
{
  ZIPChains *self = g_new0(ZIPChains, 1);
  cap_t oldcaps;
  
  oldcaps = cap_save();

  cap_enable(CAP_NET_ADMIN);
  /* cap_enable(CAP_NET_RAW); */
  self->fd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
  cap_restore(oldcaps);
  if (self->fd == -1)
    {
      z_log(NULL, CORE_ERROR, 0, "Error creating RAW socket: %m");
      g_free(self);
      return NULL;
    }
  return self;
}

gboolean
z_ipchains_add(ZIPChains *self, gchar *chain, struct ip_fwuser *r)
{
  struct ip_fwnew new;
  cap_t oldcaps;

  new.fwn_rulenum = 1;
  memcpy(&new.fwn_rule, r, sizeof(struct ip_fwuser));  
  strncpy(new.fwn_label, chain, sizeof(new.fwn_label) - 1);
  new.fwn_label[sizeof(new.fwn_label) - 1] = 0;
  oldcaps = cap_save();
  cap_enable(CAP_NET_ADMIN);
  if (setsockopt(self->fd, IPPROTO_IP, IP_FW_INSERT, &new, sizeof(new)) == -1)
    {
      cap_restore(oldcaps);
      z_log(NULL, CORE_ERROR, 0, "setsockopt failed, %m");
      return FALSE;
    }
  cap_restore(oldcaps);
  return TRUE;
}

gboolean
z_ipchains_del(ZIPChains *self, gchar *chain, struct ip_fwuser *r)
{
  struct ip_fwchange change;
  cap_t oldcaps;

  memcpy(&change.fwc_rule, r, sizeof(struct ip_fwuser));
  strncpy(change.fwc_label, chain, sizeof(change.fwc_label) - 1);
  change.fwc_label[sizeof(change.fwc_label) - 1] = 0;
  oldcaps = cap_save();
  cap_enable(CAP_NET_ADMIN);
  if (setsockopt(self->fd, IPPROTO_IP, IP_FW_DELETE, &change, sizeof(change)) == -1)
    {
      cap_restore(oldcaps);
      z_log(NULL, CORE_ERROR, 0, "setsockopt failed, %m");
      return FALSE;
    }
  cap_restore(oldcaps);
  return TRUE;
}

#if 0
static gint 
z_ipchains_redirect(ZPacketFilter *s, guint proto, 
		    ZPFDomain *src, guint sport[2], 
		    ZPFDomain *dst, guint dport[2], 
		    guint redirport)
{
  ZIPChains *self = (ZIPChains *) s;
  ZPFInetDomain *srcinet, *dstinet;
  int i;

  if (src->family != AF_INET || dst->family != AF_INET)
    {
      z_log(NULL, CORE_ERROR, 0, "Error adding redirect rule, unsupported address family %d", src->family);

      return 0;
    }

  memset(&new, 0, sizeof(new));

  srcinet = (ZPFInetDomain *) src;
  dstinet = (ZPFInetDomain *) dst;

  new.fwn_rulenum = 1;
  new.fwn_rule.ipfw.fw_src = srcinet->ip;
  new.fwn_rule.ipfw.fw_smsk = srcinet->mask;
  new.fwn_rule.ipfw.fw_dst = dstinet->ip;
  new.fwn_rule.ipfw.fw_dmsk = dstinet->mask;
  new.fwn_rule.ipfw.fw_proto = proto;
  for (i = 0; i < 2; i++)
    {
      new.fwn_rule.ipfw.fw_spts[i] = sport[i];
      new.fwn_rule.ipfw.fw_dpts[i] = dport[i];
    }
  new.fwn_rule.ipfw.fw_redirpt = redirport;
  strncpy(new.fwn_rule.label, "REDIRECT", sizeof(new.fwn_rule.label));
  strncpy(new.fwn_label, self->chain, sizeof(new.fwn_label));
  
  return TRUE;
}

static gint
z_ipchains_del(ZPacketFilter *s, guint proto, 
	       ZPFDomain *src, guint sport[2], 
	       ZPFDomain *dst, guint dport[2])
{
  ZIPChains *self = (ZIPChains *) s;
  struct ip_fwchange del;
  ZPFInetDomain *srcinet, *dstinet;
  int i;

  if (src->family != AF_INET || dst->family != AF_INET)
    {
      z_log(NULL, CORE_ERROR, 0, "Error adding redirect rule, unsupported address family %d", src->family);

      return 0;
    }

  memset(&del, 0, sizeof(del));
  srcinet = (ZPFInetDomain *) src;
  dstinet = (ZPFInetDomain *) dst;

  del.fwc_rule.ipfw.fw_src = srcinet->ip;
  del.fwc_rule.ipfw.fw_smsk = srcinet->mask;
  del.fwc_rule.ipfw.fw_dst = dstinet->ip;
  del.fwc_rule.ipfw.fw_dmsk = dstinet->mask;
  del.fwc_rule.ipfw.fw_proto = proto;
  for (i = 0; i < 2; i++)
    {
      del.fwc_rule.ipfw.fw_spts[i] = sport[i];
      del.fwc_rule.ipfw.fw_dpts[i] = dport[i];
    }
  strncpy(del.fwc_label, self->chain, sizeof(del.fwc_label));
  
  return setsockopt(self->fd, IPPROTO_IP, IP_FW_DELETE, &del, sizeof(del)) != -1;

}
#endif

void
z_ipchains_destroy(ZIPChains *self)
{
  close(self->fd);
  g_free(self);
}
