/***************************************************************************
 *
 * Copyright (c) 2000 BalaBit IT Ltd, Budapest, Hungary
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: modules.c,v 1.7 2000/12/20 16:28:30 sasa Exp $
 *
 * Author  : Bazsi
 * Auditor : kisza
 * Last version: 1.4
 * Notes:
 *
 ***************************************************************************/

#include <zorp/zorp.h>
#include <zorp/log.h>
#include <gmodule.h>


#define CORE_DEBUG "core.debug"
#define CORE_ERROR "core.error"
/*+

  Loads the given proxy module from disk, and calls its initialize
  function.

  Parameters:
    modname        module name to load without path or extension 
                   information

  Returns:
    TRUE on success

  +*/
gint
z_load_module(gchar *modname)
{
  GModule *m;
  gchar *buf;
  gint (*modinit)(void);
  
  buf = g_module_build_path(ZORP_LIBDIR, modname);
  m = g_module_open(buf, 0);
  if (m &&
      g_module_symbol(m, "zorp_module_init", (gpointer *) &modinit) &&
      modinit())
    {
      z_log(NULL, CORE_DEBUG, 8, "Module %s loaded from %s", modname, buf);
      g_free(buf);
      return TRUE;
    }
  else
    {
      z_log(NULL, CORE_ERROR, 1, "Error loading module %s, attempted from %s, error: %s", modname, buf, g_module_error());
    }
  g_free(buf);
  return FALSE;
}
