/***************************************************************************
 *
 * Copyright (c) 2000 BalaBit IT Ltd, Budapest, Hungary
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: pyreceive.c,v 1.10 2001/01/02 08:01:30 sasa Exp $
 *
 * Author  : Bazsi
 * Auditor : kisza
 * Last version: 1.6
 * Notes:
 *   - comments
 *   - process identifier in messages
 *
 * Author  : Bazsi
 * Auditor : kisza
 * Last version: 1.6
 * Notes:
 *   - comments
 *   - process identifier in messages
 *
 ***************************************************************************/

#include <zorp/pyreceive.h>
#include <zorp/receive.h>
#include <zorp/pysockaddr.h>
#include <zorp/log.h>
#include <zorp/policy.h>

/* Shamelessly copied from pylisten.c :) */

typedef struct _ZorpReceive
{
  PyObject_HEAD 
  ZIOReceive * receive;
  int fd;
} ZorpReceive;

static PyObject *z_py_zorp_receive_new_instance(PyObject *o, PyObject *args);
static void z_py_zorp_receive_free(ZorpReceive *self);
static PyObject *z_py_zorp_receive_getattr(PyObject *o, char *name);

/* static void z_py_zorp_receive_destroy(ZorpReceive *self); */
static PyObject *z_py_zorp_receive_getfd(ZorpReceive *self, PyObject *args);
static PyObject *z_py_zorp_receive_destroy_method(ZorpReceive *self, PyObject *args);

PyMethodDef z_py_zorp_receive_funcs[] = 
{
  { "Receive", z_py_zorp_receive_new_instance, METH_VARARGS },
  { NULL, NULL }			/* sentinel */
};

static PyMethodDef py_zorp_receive_methods[] = 
{
  { "getFD", (PyCFunction) z_py_zorp_receive_getfd, 0 },
  { "destroy", (PyCFunction) z_py_zorp_receive_destroy_method, 0 },
  { NULL, NULL}			/* sentinel */
};

PyTypeObject z_py_zorp_receive_type = 
{
  PyObject_HEAD_INIT(&z_py_zorp_receive_type) 0,
  "ZorpReceive",
  sizeof(ZorpReceive),
  0,
  (destructor) z_py_zorp_receive_free,
  0,
  (getattrfunc) z_py_zorp_receive_getattr,
  0,
  0,
  0,				/*(reprfunc) z_py_zorp_receive_repr, */
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  "ZorpReceive class for Zorp",
};

static void
z_py_zorp_receive_accept(int fd, gpointer user_data)
{
  ZorpReceive *self = (ZorpReceive *) user_data;

  self->fd = fd;  
}

static PyObject *
z_py_zorp_receive_new_instance(PyObject *o, PyObject *args)
{
  ZorpReceive *self;
  ZorpSockAddr *addr;
  char buf[128];

  if (!PyArg_ParseTuple(args, "O", &addr))
    return NULL;
    
  z_log(NULL, CORE_DEBUG, 7, "z_py_zorp_receive_new(), addr=%s",
	  z_sockaddr_format(addr->sa, buf, sizeof(buf)));
  
  self = PyObject_NEW(ZorpReceive, &z_py_zorp_receive_type);
  Py_XINCREF(self);
  self->receive = z_io_receive_new(addr->sa, z_py_zorp_receive_accept, self);
  if (self->receive == NULL) 
    {
      PyErr_SetString(PyExc_IOError, "Unable to install receiver");
      Py_XDECREF(self);
      return NULL;
    }

  return (PyObject *) self;
}

static void
z_py_zorp_receive_free(ZorpReceive *self)
{
  if (self->receive)
    {
      z_io_receive_destroy(self->receive);
      self->receive = NULL;
    }
  PyMem_DEL(self);
}

static PyObject *
z_py_zorp_receive_getattr(PyObject * o, char *name)
{
  return Py_FindMethod(py_zorp_receive_methods, o, name);
}

#if 0
static void
z_py_zorp_receive_destroy(ZorpReceive * self)
{
  z_policy_lock(z_python_global_state);
  Py_XDECREF(self);
  z_policy_unlock(z_python_global_state);
}
#endif 

static PyObject *
z_py_zorp_receive_getfd(ZorpReceive *self, PyObject *args)
{
  return z_policy_var_build("i", self->fd);
}

static PyObject *
z_py_zorp_receive_destroy_method(ZorpReceive *self, PyObject *args)
{
  Py_XDECREF(self);
  Py_XINCREF(Py_None);
  return Py_None;
}

void
z_py_zorp_receive_init(void)
{
  Py_InitModule("Zorp.Zorp", z_py_zorp_receive_funcs);
}
