/***************************************************************************
 *
 * Copyright (c) 2000 BalaBit IT Ltd, Budapest, Hungary
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: registry.c,v 1.10 2000/10/10 18:25:58 bazsi Exp $
 *
 * Author  : Bazsi
 * Auditor : kisza
 * Last version: 1.8
 * Notes:
 *
 * Author  : Bazsi
 * Auditor : kisza
 * Last version: 1.8
 * Notes:
 *
 ***************************************************************************/

#include <zorp/registry.h>
#include <glib.h>

static GHashTable *registry;

#define MAX_REGISTRY_NAME 32

/*+

  An entry in the registry. The registry is a hash table indexed by a
  string and storing an opaque pointer and an integer type
  value. ZRegistryEntry is an entry in the registry hash.

  +*/
typedef struct _ZRegistryEntry
{
  gint type;
  gchar name[MAX_REGISTRY_NAME];
  gpointer value;
} ZRegistryEntry;

/*+

  Initialize the registry subsystem.

  Parameters:
    none

  Returns:
    none

  +*/
void
z_registry_init(void)
{
  registry = g_hash_table_new(g_str_hash,
			      g_str_equal);
}


/*+

  Deinitialize the registry subsystem.

  Parameters:
    none

  Returns:
    none

  +*/
void
z_registry_destroy(void)
{
  g_hash_table_destroy(registry);
}

/*+

  Add an entry to the registry with the given name and type.

  Parameters:
    name       the key of this entry
    type       the type of this entry
    value      the pointer associated with this entry

  Returns:
    none

  +*/
void
z_registry_add(gchar *name, gint type, gpointer value)
{
  ZRegistryEntry *ze = g_new0(ZRegistryEntry, 1);

  strncpy(ze->name, name, sizeof(ze->name));
  ze->value = value;
  ze->type = type;
  g_hash_table_insert(registry, ze->name, ze);
}

/*+

  Fetch an item from the registry with the given name returning its
  type and pointer.

  Parameters:
    name      name of the entry to fetch
    type      pointer to an int, where the type of the entry is stored.

  Returns:
    NULL if the entry was not found, the associated pointer otherwise

  +*/
gpointer
z_registry_get(gchar *name, gint *type)
{
  ZRegistryEntry *ze = g_hash_table_lookup(registry, name);
  if (ze)
    {
      if (type) 
        *type = ze->type;
      return ze->value;
    }
  return NULL;    
}
