/***************************************************************************
 *
 * Copyright (c) 2000 BalaBit IT Ltd, Budapest, Hungary
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: packsock.h,v 1.2 2000/08/01 10:10:36 bazsi Exp $
 *
 ***************************************************************************/

#ifndef ZORP_PACKSOCK_H_INCLUDED
#define ZORP_PACKSOCK_H_INCLUDED

#include <zorp/sockaddr.h>

/*
   These packets are sent. No references to the fields from, to and data
   are left, so the receiver should free these after processing.
   Every received packet must be at most MAX_UDP_PACKET bytes long.
   The memory allocated for data is always MAX_UDP_PACKET long. Perhaps
   some shrinking should be done after receiving.
*/

typedef struct _ZPacket
{
  ZSockAddr *from;
  ZSockAddr *to;
  int length;
  char *data;
} ZPacket;

/*
   Install a default receiver on an UDP port. From now on, every received
   packet will be sent to fd as a ZPacket.
   Returns 1 for success, 0 otherwise.
*/

int z_packsock_add_default_fd(int port, int fd);

/*
   Install an one-shot packet receiver. A from->to packet on specified port
   will be sent to fd, if it arrives within timeout. port should be equal
   to to->sa.sin_port, although no check for this is made currently.
   Irreal timeouts are not allowed. Returns 1 for success, 0 otherwise.
   from and to are g_free'd after use, so do not save references to them.
   Also, do not g_free them yourself.
*/

int z_packsock_add_special_fd(int port,
			      ZSockAddr * from, ZSockAddr * to, int timeout,
			      int fd);

/*
   Kills the default handler on port. This does not mean that listening
   does not continue on that port, because one-shot handler may be still
   active.
*/

void z_packsock_close(int port);

/*
   Forges an UDP packet from zfrom to zto. Uses kernel option IP: transparent
   proxying. Nothing is done to ZSockAddr's, so it is your responsibility
   to free them.
*/

int z_packsock_sendfrom(ZSockAddr * zfrom, ZSockAddr * zto, char *buf,
			int len);

#endif
