/***************************************************************************
 *
 * Copyright (c) 2000 BalaBit IT Ltd, Budapest, Hungary
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: ftpcmd.c,v 1.15 2001/01/09 09:41:08 sasa Exp $
 *
 * Author:  Andras Kis-Szabo <kisza@sch.bme.hu>
 * Author:  Attila SZALAY <sasa@balabit.hu>
 * Auditor:
 * Notes:
 *
 ***************************************************************************/

#include <zorp/zorp.h>
#include <zorp/stream.h>
#include <zorp/readline.h>
#include <zorp/proxy.h>
#include <zorp/policy.h>
#include <zorp/thread.h>
#include <zorp/zpython.h>
#include <zorp/log.h>

#include "ftp.h"
#include "ftphash.h"
#include "ftpcmd.h"

#include <assert.h>
#include <arpa/inet.h>

gboolean
ftp_parse_nums(gchar *src, gint length, unsigned char *nums)
{
  int i = 0;
  gchar *newsrc;
  
  while (length > 0 && i < 6)
    {
      unsigned int tmp;
      
      errno = 0;
      tmp = strtoul(src, &newsrc, 10);
      if (tmp > 255 || errno == ERANGE)
        {
          return FALSE;
        }
      nums[i] = tmp;
      if (i < 5 && *newsrc != ',')
        {
          return FALSE;
        }
      length -= (newsrc - src + 1);
      src = newsrc + 1;
      i++;
    }
  if (length <= 0)
    return TRUE;
  else
    return FALSE;
}

gboolean
ftp_parse_search_nums(gchar *src, gint length, unsigned char *nums)
{
  gchar *left, *right;

  left = strchr(src,'(');
  if (left)
    {
      right = strrchr(src,')');
      if (right)
        {
          left++;
          length = right - left;
          if (length > 0)
            return (ftp_parse_nums(left, length, nums));
        }
    }
  return FALSE;
}
      

// Function:    ftp_command_parse_USER
// In:          FtpProxy                Proxy pointer
//              FtpCommandDescriptor    found command
//              char**                  sended line
// Out:         int
// Desc:        USER command
guint
ftp_command_parse_USER(FtpProxy *self, FtpInternalCommand *command)
{
  gchar *atsign;
    
  z_proxy_log(self, "ftp.debug", 7, "ftp_command_parse_USER called");

  switch (self->ftp_state)
    {
    case FTP_STATE_PRECONNECT:
    case FTP_STATE_PRECONNECT_LOGIN:
    case FTP_STATE_PRECONNECT_LOGIN_U:
    case FTP_STATE_PRECONNECT_LOGIN_P:
      /* USER in non transparent mode, we'll need to find out destination hostname */
      
      /* 
       * FIXME: there should be other ways of telling the proxy where to
       * connect, like SITE HOST <hostname> 
       */ 

      z_proxy_log(self, "ftp.debug", 8, "USER command: %s state: %d", self->command_param, self->ftp_state);
       
      atsign = strrchr(self->command_param, '@');
      if (!atsign)
        {
          SET_ANSWER(MSG_USERNAME_FORMAT_INVALID);
          z_proxy_log(self, "ftp.debug", 6, "USER command: cannot found @ sign in non-transparent mode");
          return FTP_CMD_DROP;
        }
        
      if (strlen(atsign + 1) > self->max_hostname_length)
        {
          SET_ANSWER(MSG_HOSTNAME_TOO_LONG);
          z_proxy_log(self, "ftp.debug", 6, "USER command: hostname too long");
          return FTP_CMD_DROP;
        }

      *atsign = 0;
      if (strlen(self->command_param) > self->max_username_length)
        {
          SET_ANSWER(MSG_USERNAME_TOO_LONG);
          z_proxy_log(self, "ftp.debug", 6, "USER command: username too long");
          return FTP_CMD_DROP;
        }
        
      /* 
       * NOTE: the length of hostname and username is checked above, this is
       * not BOF 
       */
      strcpy(self->hostname, atsign + 1);
      strcpy(self->username, self->command_param);
      self->ftp_state = FTP_STATE_PRECONNECT_LOGIN_U;

      SET_ANSWER(MSG_USER_OKAY);
      z_proxy_log(self, "ftp.debug", 7, "USER command: Login ok.");
      return FTP_PROXY_ANS;
    case FTP_STATE_LOGIN:
    case FTP_STATE_LOGIN_U:
    case FTP_STATE_LOGIN_P:
    case FTP_STATE_LOGIN_A:
    case FTP_STATE_LOGINAUTH:
      if (strlen(self->command_param) > self->max_username_length)
        {
          SET_ANSWER(MSG_USERNAME_TOO_LONG);
          return FTP_CMD_DROP;
        }
      strcpy(self->username, self->command_param);
      self->ftp_state = FTP_STATE_LOGIN_U;
      break;
    case FTP_STATE_CONVERSATION:
      SET_ANSWER(MSG_USER_ALREADY_LOGGED_IN);
      return FTP_CMD_DROP;
    default:
      return FTP_CMD_ABORT;
    }
  return FTP_CMD_OK;
}

guint
ftp_command_answer_USER(FtpProxy *self, FtpInternalCommand *cmd)
{
  switch (self->ftp_state)
    {
    case FTP_STATE_LOGIN_U:
      switch(self->answer[0])
        {
        case '2':
          self->ftp_state = FTP_STATE_CONVERSATION;
          break;
        case '3':
          switch(self->answer_code)
            {
            case 331:
              break;
            case 332:
              /* TODO: handling account */
            }
          break;
        }
      break;
    }
  return FTP_ANS_OK;
}

// Function:    ftp_command_parse_PASS
// In:          FtpProxy                Proxy pointer
//              FtpCommandDescriptor    found command
//              char**                  sended line
// Out:         int
// Desc:        PASS command
guint
ftp_command_parse_PASS(FtpProxy * self, FtpInternalCommand * command)
{
  switch (self->ftp_state)
    {
    case FTP_STATE_PRECONNECT:
    case FTP_STATE_PRECONNECT_LOGIN:
    case FTP_STATE_PRECONNECT_LOGIN_P:

      /* PASS in non-transparent startup */
      SET_ANSWER(MSG_USER_FIRST);
      return FTP_CMD_DROP;
    case FTP_STATE_PRECONNECT_LOGIN_U:
      if (strlen(self->command_param) > self->max_password_length)
        {
          SET_ANSWER(MSG_PASSWORD_TOO_LONG);
          return FTP_CMD_DROP;
        }
      strcpy(self->password, self->command_param);
      self->ftp_state = FTP_STATE_PRECONNECT_LOGIN_P;
      return FTP_CMD_NOOP;

    case FTP_STATE_LOGIN:
    case FTP_STATE_LOGIN_P:
    case FTP_STATE_LOGIN_A:
    case FTP_STATE_LOGINAUTH:
      SET_ANSWER(MSG_USER_FIRST);
      return FTP_CMD_DROP;
    case FTP_STATE_LOGIN_U:
      if (strlen(self->command_param) > self->max_password_length)
        {
          SET_ANSWER(MSG_PASSWORD_TOO_LONG);
          return FTP_CMD_DROP;
        }
      strcpy(self->password, self->command_param);
      self->ftp_state = FTP_STATE_LOGIN_P;
      break;
    case FTP_STATE_CONVERSATION:
      SET_ANSWER(MSG_USER_FIRST);
      return FTP_CMD_DROP;
    default:
      return FTP_CMD_ABORT;
    }
  return FTP_CMD_OK;
}

guint
ftp_command_answer_PASS(FtpProxy *self, FtpInternalCommand *cmd)
{
  switch (self->ftp_state)
    {
    case FTP_STATE_LOGIN_P:
      switch (self->answer[0])
        {
        case '2':
          self->ftp_state = FTP_STATE_CONVERSATION;
          break;
        case '3':
          switch(self->answer_code)
            {
            case 332:
              /* TODO: handling account */
              break;
            }
          break;
        }
      break;
    }
  return FTP_ANS_OK;
}


guint
ftp_command_parse_path(FtpProxy *self, FtpInternalCommand *command)
{
  switch (self->ftp_state)
    {
    case FTP_STATE_CONVERSATION:
      if (command->need_data)
        self->state=FTP_BOTH_SIDE;
      break;
    default:
      ftp_listen_both_side_stop(self);
      SET_ANSWER(MSG_COMMAND_NOT_ALLOWED_HERE);
      return FTP_CMD_DROP;
    }
  return FTP_CMD_OK;
}

guint
ftp_command_answer_path(FtpProxy *self, FtpInternalCommand *command)
{
  if (!command->need_data)
    return FTP_ANS_OK;
  switch (self->ftp_state)
    {
    case FTP_STATE_CONVERSATION:
      switch (self->answer[0])
        {
        case '1':
          self->data_state|=FTP_DATA_SERVER_SAID;
          ftp_data_next_step(self);
          break;
        case '2':
          self->oldstate=FTP_CLIENT_TO_SERVER;
          break;
        case '4':
        case '5':
          self->data_state = 0;
          self->oldstate = FTP_CLIENT_TO_SERVER;
          break;
        default:
          z_proxy_log(self, "ftp.error", 2, "Unknown answer: %d. Abort (data)connection!",self->answer_code);
          self->data_state=0;
          self->oldstate = FTP_CLIENT_TO_SERVER;
          break;
        }
      break;
    default:
      SET_ANSWER(MSG_COMMAND_NOT_ALLOWED_HERE);
      return FTP_ANS_CHANGE;
    }
  return FTP_ANS_OK;
}

// Function:    ftp_command_parse_QUIT
// In:          FtpProxy                Proxy pointer
//              FtpCommandDescriptor    found command
//              gchar **		message
// Out:         int
// Desc:        QUIT command
guint
ftp_command_parse_QUIT(FtpProxy *self, FtpInternalCommand *command)
{
  switch (self->ftp_state)
    {
    case FTP_STATE_PRECONNECT:
    case FTP_STATE_PRECONNECT_LOGIN_U:
    case FTP_STATE_PRECONNECT_LOGIN_P:
      if (strlen(self->command_param) > 0)
        {
          SET_ANSWER(MSG_INVALID_PARAMETER);
          return FTP_CMD_DROP;
        }
      SET_ANSWER(MSG_GOODBYE);
      self->ftp_state = FTP_STATE_PRECONNECT_QUIT;
      return FTP_CMD_DROP;
    case FTP_STATE_LOGIN:
    case FTP_STATE_LOGIN_U:
    case FTP_STATE_LOGIN_P:
    case FTP_STATE_LOGIN_A:
    case FTP_STATE_LOGINAUTH:
    case FTP_STATE_CONVERSATION:
      if (strlen(self->command_param) > 0)
        {
          SET_ANSWER(MSG_INVALID_PARAMETER);
          return FTP_CMD_DROP;
        }
      ftp_listen_both_side_stop(self);
      self->ftp_state = FTP_STATE_QUIT;
      break;
    default:
      return FTP_CMD_DROP;
    }
  return FTP_CMD_OK;
}

guint
ftp_command_answer_QUIT(FtpProxy *self, FtpInternalCommand *command)
{
  switch (self->ftp_state)
    {
    case FTP_STATE_QUIT:
      self->state = FTP_QUIT;
      break;
    }
  return FTP_ANS_OK;
}

guint
ftp_command_parse_TYPE(FtpProxy *self, FtpInternalCommand *command)
{
  if(self->ftp_state==FTP_STATE_CONVERSATION)
    {

      if (strlen(self->command_param)==0)
        {
          SET_ANSWER(MSG_MISSING_PARAMETER);
          return FTP_CMD_DROP;
        }

      switch(self->command_param[0])
        {
        case 'a':
        case 'A':
        case 'i':
        case 'I':
          self->command_param[1]=0;
          break;
        case 'l':
        case 'L':
        case 'e':
        case 'E':
          SET_ANSWER(MSG_COMMAND_NOT_IMPLEMENTED);
          return FTP_CMD_DROP;
        default:
          SET_ANSWER(MSG_COMMAND_NOT_RECOGNIZED);
          return FTP_CMD_DROP; 
        }

    }
  else
    {
      SET_ANSWER(MSG_COMMAND_NOT_ALLOWED_HERE);
      return FTP_CMD_DROP;
    }
  return FTP_CMD_OK;
}

guint
ftp_command_parse_ABOR(FtpProxy *self, FtpInternalCommand *command)
{
  char buf[3];

  buf[0]=0xff;
  buf[1]=0xf4;
  buf[2]=0xff;

  if(self->ftp_state==FTP_STATE_CONVERSATION)
    {
      z_stream_send(self->super.endpoints[EP_SERVER], buf, 3, MSG_OOB);
      buf[0]=0xf2;
      z_stream_send(self->super.endpoints[EP_SERVER], buf, 1, 0);
      self->state = FTP_SERVER_TO_CLIENT;
      return FTP_CMD_OK;
    }
  else if (self->ftp_state==FTP_STATE_RENAME)
    {
      self->ftp_state = FTP_STATE_CONVERSATION;
    }

  SET_ANSWER(MSG_COMMAND_NOT_ALLOWED_HERE);
  return FTP_CMD_DROP;
}

guint
ftp_command_answer_ABOR(FtpProxy *self, FtpInternalCommand *command)
{
  if(self->ftp_state==FTP_STATE_CONVERSATION)
    {

      if(self->answer[0]=='2')
        {
          ftp_listen_both_side_stop(self);
          self->oldstate = FTP_CLIENT_TO_SERVER;
        }
      else if(self->answer[0]=='4')
        {
          self->oldstate = FTP_SERVER_TO_CLIENT;
          self->data_state=0;
        }

    }
  return FTP_ANS_OK;
}

guint
ftp_command_parse_noarg(FtpProxy *self, FtpInternalCommand *command)
{
  switch (self->ftp_state)
  {
  case FTP_STATE_CONVERSATION:
    self->command_param[0]=0;
    break;
  default:
    SET_ANSWER(MSG_COMMAND_NOT_ALLOWED_HERE);
    return FTP_CMD_DROP;
  }
  return FTP_CMD_OK;
}

guint
ftp_command_parse_HELP(FtpProxy *self, FtpInternalCommand *command)
{
  return FTP_CMD_OK;
}

guint
ftp_command_parse_MODE(FtpProxy *self, FtpInternalCommand *command)
{
  if(self->ftp_state==FTP_STATE_CONVERSATION)
    {

      if (strlen(self->command_param)==0)
        {
          SET_ANSWER(MSG_MISSING_PARAMETER);
          return FTP_CMD_DROP;
        }

      switch(self->command_param[0])
        {
        case 's':
        case 'S':
        case 'b':
        case 'B':
        case 'c':
        case 'C':
          self->command_param[1]=0;
          break;
        default:
          SET_ANSWER(MSG_COMMAND_NOT_RECOGNIZED);
          return FTP_CMD_DROP; 
        }
    }
  else
    {
      SET_ANSWER(MSG_COMMAND_NOT_ALLOWED_HERE);
      return FTP_CMD_DROP;
    }
  return FTP_CMD_OK;
}

guint
ftp_command_parse_string(FtpProxy *self, FtpInternalCommand *command)
{
  switch (self->ftp_state)
    {
    case FTP_STATE_CONVERSATION:
      break;
    default:
      SET_ANSWER(MSG_COMMAND_NOT_ALLOWED_HERE);
      return FTP_CMD_DROP;
    }
  return FTP_CMD_OK;
}

guint
ftp_command_parse_STRU(FtpProxy *self, FtpInternalCommand *command)
{
  if(self->ftp_state==FTP_STATE_CONVERSATION)
    {

      if (strlen(self->command_param)==0)
        {
          SET_ANSWER(MSG_MISSING_PARAMETER);
          return FTP_CMD_DROP;
        }

      switch(self->command_param[0])
        {
        case 's':
        case 'S':
        case 'b':
        case 'B':
        case 'c':
        case 'C':
          self->command_param[1]=0;
          break;
        default:
          SET_ANSWER(MSG_COMMAND_NOT_RECOGNIZED);
          return FTP_CMD_DROP; 
        }
    }
  else
    {
      SET_ANSWER(MSG_COMMAND_NOT_ALLOWED_HERE);
      return FTP_CMD_DROP;
    }
  return FTP_CMD_OK;
}

guint
ftp_data_server_start_active(FtpProxy *self)
{
  guint port, length;

  self->data_state|=FTP_DATA_SERVER_START;

  if (!ftp_data_prepare(self, FTP_SIDE_SERVER, 'L'))
    {
      SET_ANSWER(MSG_ERROR_PARSING_PORT);
      self->data_state = 0;
      return FTP_CMD_DROP;
    }

  z_inet_ntoa(self->command_param, self->max_line_length,((struct sockaddr_in *) &self->fw_server_data->sa)->sin_addr);
  g_strdelimit(self->command_param, ".", ',');

  port = ntohs(((struct sockaddr_in *) &self->fw_server_data->sa)->sin_port);
  if(port==0)
    {
      SET_ANSWER(MSG_ERROR_PARSING_PORT);

      ftp_listen_both_side_stop(self);
      return FTP_CMD_DROP;
    }

  length = strlen(self->command_param);
  g_snprintf(self->command_param + length,self->max_line_length - length,",%d,%d", (port & 0xff00) >> 8, port & 0x00ff);
  return FTP_CMD_OK;
}

guint
ftp_data_server_start_passive(FtpProxy *self)
{
  guchar nums[6], ip[17];

  if (!ftp_parse_search_nums(self->answer_param, strlen(self->answer_param), nums))
    {
      SET_ANSWER(MSG_ERROR_PARAMETER_PASV);
      return FTP_ANS_CHANGE;
    }

  g_snprintf(ip, sizeof(ip), "%d.%d.%d.%d", nums[0], nums[1], nums[2], nums[3]);
  assert((self->server_data->sa.sa_family == AF_INET));
  inet_aton(ip, &((struct sockaddr_in *) &self->server_data->sa)->sin_addr);
  ((struct sockaddr_in *) &self->server_data->sa)->sin_port = htons(nums[4] * 256 + nums[5]);

  self->data_state|=FTP_DATA_SERVER_START;

  if (!ftp_data_prepare(self, FTP_SIDE_SERVER, 'C'))
    {
      SET_ANSWER(MSG_ERROR_PARSING_PASV);
      self->data_state = 0;
      return FTP_ANS_CHANGE;
    }
  return FTP_ANS_OK;
}

guint
ftp_command_parse_PORT(FtpProxy *self, FtpInternalCommand *command)
{
  guchar nums[6], ip[17];
  guint vissza=FTP_CMD_OK;

  switch (self->ftp_state)
    {
    case FTP_STATE_CONVERSATION:
      if (!ftp_parse_nums(self->command_param, strlen(self->command_param), nums))
        {
          SET_ANSWER(MSG_ERROR_PARAMETER_PORT);
          return FTP_CMD_DROP;
        }

      g_snprintf(ip, sizeof(ip), "%d.%d.%d.%d", nums[0], nums[1], nums[2], nums[3]);
      assert((self->client_data->sa.sa_family == AF_INET));

      self->data_state=0;

      inet_aton(ip, &((struct sockaddr_in *) &self->client_data->sa)->sin_addr);
      ((struct sockaddr_in *) &self->client_data->sa)->sin_port = htons(nums[4] * 256 + nums[5]);

      ftp_listen_both_side_start(self);

      switch (self->data_mode)
	{
	case FTP_DATA_PASSIVE:
	   strcpy(self->command,"PASV");
	   self->command_param[0]=0;
	   break;
	case FTP_DATA_ACTIVE:
	case FTP_DATA_KEEP:
	  vissza=ftp_data_server_start_active(self);
	  break;
	default:
	  ftp_listen_both_side_stop(self);
	  z_proxy_log(self, "ftp.error", 2, "connection mode not supported, data_mode=%d", self->data_mode);
	  SET_ANSWER(MSG_ERROR_PARSING_PORT);
	  return FTP_CMD_DROP;
	}
      break;
    default:
      SET_ANSWER(MSG_COMMAND_NOT_ALLOWED_HERE);
      return FTP_CMD_DROP;
    }
  return vissza;
}

guint ftp_command_answer_PORT(FtpProxy *self, FtpInternalCommand *command)
{
  guint vissza = FTP_ANS_OK;
  switch (self->ftp_state)
    {
    case FTP_STATE_CONVERSATION:
      switch (self->data_mode)
        {
        case FTP_DATA_PASSIVE:
          switch (self->answer[0])
            {
            case '2':
              vissza=ftp_data_server_start_passive(self);
              if(vissza==FTP_ANS_OK)
                {
                  ((struct sockaddr_in *) &self->fw_client_data->sa)->sin_port = htons(20);

                  if (!ftp_data_prepare(self, FTP_SIDE_CLIENT, 'C'))
                    {
                      self->data_state=0;
                      SET_ANSWER(MSG_ERROR_PARSING_PORT);
                      return FTP_ANS_CHANGE;
                    }

                  SET_ANSWER(MSG_PORT_SUCCESFULL);
                  vissza=FTP_ANS_CHANGE;
                }
              break;
            default:
                {
                  SET_ANSWER(MSG_ERROR_PARSING_PORT);
                  self->data_state=0;
                  return FTP_ANS_CHANGE;
                }
            }
          break;
        case FTP_DATA_ACTIVE:
        case FTP_DATA_KEEP:
          switch (self->answer[0])
            {
            case '2':
              ((struct sockaddr_in *) &self->fw_client_data->sa)->sin_port = htons(20);

              if (!ftp_data_prepare(self, FTP_SIDE_CLIENT, 'C'))
                {
                  self->data_state=0;
                  SET_ANSWER(MSG_ERROR_PARSING_PORT);
                  return FTP_ANS_CHANGE;
                }
              break;
            default:
                self->data_state=0;
                return FTP_ANS_OK;
            }
          break;
        default:
          break;
        }
    }
  return vissza;
}

guint
ftp_command_parse_PASV(FtpProxy *self, FtpInternalCommand *command)
{
  guint vissza = FTP_CMD_OK;

  switch (self->ftp_state)
    {
    case FTP_STATE_CONVERSATION:
      self->command_param[0]=0;
      self->data_state=0;

      ftp_listen_both_side_start(self);

      switch (self->data_mode)
        {
        case FTP_DATA_KEEP:
        case FTP_DATA_PASSIVE:
          break;
        case FTP_DATA_ACTIVE:
          strcpy(self->command,"PORT");
          self->command_param[0]=0;
          vissza=ftp_data_server_start_active(self);
          break;
        default:
          ftp_listen_both_side_stop(self);
          z_proxy_log(self, "ftp.error", 2, "connection mode not supported, data_mode=%d", self->data_mode);
          SET_ANSWER(MSG_ERROR_PARSING_PORT);
          return FTP_CMD_DROP;
        }
      break;
    default:
      SET_ANSWER(MSG_COMMAND_NOT_ALLOWED_HERE);
      return FTP_CMD_DROP;
    }
  return FTP_CMD_OK;
}

guint ftp_command_answer_PASV(FtpProxy *self, FtpInternalCommand *command)
{
  gchar *kezdo;
  guint port, length;
  guint vissza=FTP_ANS_OK;

  switch (self->ftp_state)
    {
    case FTP_STATE_CONVERSATION:
      switch (self->data_mode)
        {
        case FTP_DATA_PASSIVE:
        case FTP_DATA_KEEP:
          switch (self->answer[0])
            {
            case '2':
              vissza=ftp_data_server_start_passive(self);
              if(vissza==FTP_ANS_OK)
                {
                  ((struct sockaddr_in *) &self->fw_client_data->sa)->sin_port = htons(0);

                  if (!ftp_data_prepare(self, FTP_SIDE_CLIENT, 'L'))
                    {
                      self->data_state=0;
                      SET_ANSWER(MSG_ERROR_PARSING_PASV);
                      return FTP_ANS_CHANGE;
                    }

                  kezdo=strchr(self->answer_param,'(');
                  if(kezdo)
                    {
                      kezdo++;
                      length = kezdo-self->answer_param;
                      z_inet_ntoa(kezdo, self->max_line_length-length,((struct sockaddr_in *) &self->fw_client_data->sa)->sin_addr);
                      g_strdelimit(kezdo, ".", ',');

                      port = ntohs(((struct sockaddr_in *) &self->fw_client_data->sa)->sin_port);
                      if(port==0)
                        {
                          SET_ANSWER(MSG_ERROR_PARSING_PASV);
                          self->data_state=0;
                          return FTP_ANS_CHANGE;
                        }

                      length = strlen(self->answer_param);
                      g_snprintf(self->answer_param + length,self->max_line_length - length,",%d,%d", (port & 0xff00) >> 8, port & 0x00ff);
                      length = strlen(self->answer_param);
                      self->answer_param[length]=')';
                      self->answer_param[length+1]='.';
                      self->answer_param[length+2]=0;
                    }
                }
              break;
            default:
              self->data_state=0;
              return FTP_ANS_OK;
            }
          break;
        case FTP_DATA_ACTIVE:
          switch (self->answer[0])
            {
            case '2':
              ((struct sockaddr_in *) &self->fw_client_data->sa)->sin_port = htons(0);

              if (!ftp_data_prepare(self, FTP_SIDE_CLIENT, 'L'))
                {
                  self->data_state=0;
                  SET_ANSWER(MSG_ERROR_PARSING_PASV);
                  return FTP_ANS_CHANGE;
                }

              strcpy(self->answer,"227");
              strcpy(self->answer_param,"Entering Passive mode (");
              length=strlen(self->answer_param);
              kezdo=self->answer_param+length;
              z_inet_ntoa(kezdo, self->max_line_length-length,((struct sockaddr_in *) &self->fw_client_data->sa)->sin_addr);
              g_strdelimit(kezdo, ".", ',');

              port = ntohs(((struct sockaddr_in *) &self->fw_client_data->sa)->sin_port);
              if(port==0)
                {
                  SET_ANSWER(MSG_ERROR_PARSING_PASV);
                  self->data_state=0;
                  return FTP_ANS_CHANGE;
                }

              length = strlen(self->answer_param);
              g_snprintf(self->answer_param + length,self->max_line_length - length,",%d,%d", (port & 0xff00) >> 8, port & 0x00ff);
              length = strlen(self->answer_param);
              self->answer_param[length]=')';
              self->answer_param[length+1]='.';
              self->answer_param[length+2]=0;
              break;
            default:
              SET_ANSWER(MSG_ERROR_PARSING_PASV);
              self->data_state=0;
              return FTP_ANS_CHANGE;
            }
          break;
        default:
          break;
        }
    }
  return vissza;
}

guint
ftp_command_answer_RNFR(FtpProxy *self, FtpInternalCommand *cmd)
{
  switch (self->ftp_state)
    {
    case FTP_STATE_CONVERSATION:
      switch(self->answer_code)
        {
          case 350:
            self->ftp_state = FTP_STATE_RENAME;
          break;
        }
      break;
    }
  return FTP_ANS_OK;
}

guint
ftp_command_parse_RNTO(FtpProxy *self, FtpInternalCommand *cmd)
{
  guint vissza = FTP_CMD_OK;
  switch (self->ftp_state)
    {
    case FTP_STATE_RENAME:
      self->ftp_state = FTP_STATE_CONVERSATION;
      vissza = ftp_command_parse_path(self,cmd);
      break;
    default:
      SET_ANSWER(MSG_RNFR_RNTO);
      return FTP_CMD_DROP;
    }
  return vissza;
}
