/***************************************************************************
 *
 * Copyright (c) 2000 BalaBit IT Ltd, Budapest, Hungary
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: ftpolicy.c,v 1.8 2000/12/15 13:48:09 sasa Exp $
 *
 * Author:  Andras Kis-Szabo <kisza@sch.bme.hu>
 * Author:  Attila SZALAY <sasa@balabit.hu>
 * Auditor:
 * Notes:
 *
 ***************************************************************************/

#include <zorp/zorp.h>
#include <zorp/stream.h>
#include <zorp/readline.h>
#include <zorp/proxy.h>
#include <zorp/policy.h>
#include <zorp/thread.h>
#include <zorp/zpython.h>
#include <zorp/log.h>

#include <string.h>
#include <ctype.h>
#include <stdlib.h>

#include "ftp.h"
#include "ftpolicy.h"


GHashTable *
ftp_policy_command_hash_create(void)
{
  GHashTable *tmp;

  tmp = g_hash_table_new(g_str_hash, g_str_equal);

  return tmp;
}

guint
ftp_policy_command_hash_search(FtpProxy *self,gchar *command)
{
  FtpCommandDescriptor *tmp=g_hash_table_lookup(self->policy_command_hash, command);
  return tmp!=NULL;
}

gboolean
ftp_hash_get_type(ZPolicyObj *tuple, guint *filter_type)
{
  ZPolicyObj *tmp;

  if (!z_policy_seq_check(tuple))
    {
      if (z_policy_var_parse(tuple, "i", filter_type))
        return TRUE;
      /* not a sequence */
      return FALSE;
    }

  tmp = z_policy_seq_getitem(tuple, 0);
  if (!z_policy_var_parse(tmp, "i", filter_type))
    {
      /* policy syntax error */
      z_policy_var_unref(tmp);
      return FALSE;
    }
  z_policy_var_unref(tmp);
  return TRUE;
}
                                                          

guint
ftp_policy_command_hash_do(FtpProxy *self)
{
  guint vissza;
  ZPolicyObj *res;
  ZPolicyObj *tmp=g_hash_table_lookup(self->policy_command_hash, self->command);
  ZPolicyObj *command_where;
  unsigned int command_do;
  if(!tmp)
    {
      tmp=g_hash_table_lookup(self->policy_command_hash, "*");
    }
  if(!tmp)
    {
      z_proxy_log(self, "ftp.debug", 6, "Policy doesn't contain default. Use hard-coded.");
      return FTP_CMD_DROP;
    }
  z_policy_lock(self->super.thread);
  if(!ftp_hash_get_type(tmp,&command_do))
    {
      z_proxy_log(self, "ftp.policy", 1, "Policy type invalid!");
    }
  z_policy_unlock(self->super.thread);
  switch(command_do)
    {
    case FTP_COMMAND_DENY:
      vissza = FTP_CMD_DROP;
      break;
    case FTP_COMMAND_ACCEPT:
      vissza = FTP_CMD_OK;
      break;
    case FTP_COMMAND_POLICY:
      z_policy_lock(self->super.thread);
      if(!z_policy_var_parse(tmp,"(iO)",&command_do,&command_where))
        {
          z_proxy_log(self, "ftp.policy", 1, "Cannot parse policy line for command: %s",self->command);
          vissza = FTP_CMD_ABORT;
        }
      else
        {
          g_string_assign(self->request_c,self->command);
          g_string_assign(self->request_p,self->command_param);
          
          g_string_assign(self->answer_c,"500");
          g_string_assign(self->answer_p,"Error parsing command");
          
          res = z_policy_call_object(command_where, z_policy_var_build("(s)",self->command));
          if (res == NULL)
            {
              z_proxy_log(self, "ftp.policy", 1,"Error in policy calling for command: %s", self->command);
              vissza = FTP_CMD_ABORT;
            }
          else
            {
              if(!z_policy_var_parse(res,"i",&vissza))
                {
                  z_proxy_log(self, "ftp.policy", 1, "Can't parsing return code for command: %s", self->command);
                  vissza = FTP_CMD_ABORT;
                }
              else
                {
                  switch(vissza)
                    {
                    case Z_OK:
                      if(strcmp(self->command_param,self->request_p->str))
                        {
                          strncpy(self->command_param,self->request_p->str,self->max_line_length-1);
                          self->command_param[self->max_line_length-1]=0;
                        }
                      vissza=FTP_CMD_OK;
                      break;
                    case Z_UNSPEC:
                    case Z_DENY:
                    case Z_REJECT:
                    case Z_DROP:
                      strncpy(self->answer,self->answer_c->str,3);
                      self->answer[3]=0;
                      
                      strncpy(self->answer_param,self->answer_p->str,self->max_line_length-1);
                      self->answer_param[self->max_line_length-1]=0;
                      
                      vissza=FTP_CMD_DROP;
                    default:
                    case Z_ABORT:
                      strncpy(self->answer,self->answer_c->str,3);
                      self->answer[3]=0;
                      
                      strncpy(self->answer_param,self->answer_p->str,self->max_line_length-1);
                      self->answer_param[self->max_line_length-1]=0;
                      
                      vissza=FTP_CMD_ABORT;
                      break;
                    }
                }
            }
        }
      z_policy_unlock(self->super.thread);
      break;
    case FTP_COMMAND_ABORT:
    default:
      vissza = FTP_CMD_ABORT;
      break;
    }
  return vissza;
}

GHashTable *
ftp_policy_answer_hash_create(void)
{
  GHashTable *tmp;

  tmp = g_hash_table_new(g_str_hash, g_str_equal);

  return tmp;
}

guint
ftp_policy_answer_hash_do(FtpProxy *self)
{
  guint vissza;
  ZPolicyObj *res;
  ZPolicyObj *tmp;
  ZPolicyObj *answer_where;
  unsigned int answer_do;
  gchar work[10];
  gchar *msg;
  int i;
  
  if (strlen(self->command))
    g_snprintf(work, sizeof(work), "%s%s", self->command, self->answer);
  else
    g_snprintf(work, sizeof(work), "Null%s", self->answer);
    
  tmp=g_hash_table_lookup(self->policy_answer_hash, work);
  
  if(!tmp)
    {
      work[strlen(work)-1]=0;
      tmp=g_hash_table_lookup(self->policy_answer_hash, work);
    }
    
  if(!tmp)
    {
      work[strlen(work)-1]=0;
      tmp=g_hash_table_lookup(self->policy_answer_hash, work);
    }
    
  if(!tmp)
    {
      work[strlen(work)-1]=0;
      tmp=g_hash_table_lookup(self->policy_answer_hash, work);
    }
    
  if(!tmp)
    {
      g_snprintf(work, sizeof(work), "*%s", self->answer);
      work[strlen(work)]=0;
      tmp=g_hash_table_lookup(self->policy_answer_hash, work);
    }
    
  if(!tmp)
    {
      work[strlen(work)-1]=0;
      tmp=g_hash_table_lookup(self->policy_answer_hash, work);
    }
    
  if(!tmp)
    {
      work[strlen(work)-1]=0;
      tmp=g_hash_table_lookup(self->policy_answer_hash, work);
    }
    
  if(!tmp)
    {
      strcpy(work,"*");
      tmp=g_hash_table_lookup(self->policy_answer_hash, work);
    }
    
  if(!tmp)
    {
      z_proxy_log(self, "ftp.debug", 6, "Policy doesn't conatin default. Use hard-coded default.");
      return FTP_ANS_CHANGE;
    }
    
  z_policy_lock(self->super.thread);
  if(!ftp_hash_get_type(tmp,&answer_do))
    {
      z_proxy_log(self, "ftp.policy", 1, "Answer type invalid for command: %s , answer: %s",self->command, self->answer);
    }
  z_policy_unlock(self->super.thread);
  
  switch(answer_do)
    {
    case FTP_ANSWER_CHANGE:
      vissza = FTP_ANS_CHANGE;
      
      z_policy_lock(self->super.thread);
      if(!z_policy_var_parse(tmp,"(is)",&answer_do,&msg))
        {
          z_proxy_log(self, "ftp.policy", 1, "Bad policy request for command: %s, answer: %s.",self->command,self->answer);
          g_string_assign(self->answer_c,"500");
          g_string_assign(self->answer_p,"Error parsing answer (bad policy)");
          vissza = FTP_ANS_ABORT;
        }
      else
        {
          for(i=0;i<3;i++)
            work[i]=msg[i];
          work[i]=0;
          g_string_assign(self->answer_c,work);
          g_string_assign(self->answer_p,&msg[i+1]);
        }
      break;
    case FTP_ANSWER_ACCEPT:
      vissza = FTP_ANS_OK;
      break;
    case FTP_ANSWER_POLICY:
      z_policy_lock(self->super.thread);
      if(!z_policy_var_parse(tmp,"(iO)",&answer_do,&answer_where))
        {
          z_proxy_log(self, "ftp.policy", 1,"Bad policy line for command: %s, answer: %s.", self->command, self->answer);
          g_string_assign(self->answer_c,"500");
          g_string_assign(self->answer_p,"Error parsing answer (bad policy)");
          vissza = FTP_ANS_ABORT;
        }
      else
        {
          g_string_assign(self->answer_c,self->answer);
          g_string_assign(self->answer_p,self->answer_param);
          res = z_policy_call_object(answer_where, z_policy_var_build("(ss)",self->command,self->answer));
          if (res == NULL)
            {
              z_proxy_log(self, "ftp.policy", 1, "Error in policy calling for command: %s, answer: %s.",self->command, self->answer);
              g_string_assign(self->answer_c,"500");
              g_string_assign(self->answer_p,"Error parsing answer (bad policy)");
              vissza = FTP_ANS_ABORT;
            }
          else
            {
              if(!z_policy_var_parse(res,"i",&vissza))
                {
                  z_proxy_log(self, "ftp.policy", 1, "Return code invalid from policy function for command: %s, answer: %s",self->command, self->answer);
                  g_string_assign(self->answer_c,"500");
                  g_string_assign(self->answer_p,"Error parsing answer (bad policy)");
                  vissza = FTP_ANS_ABORT;
                }
              else
                {
                  switch(vissza)
                    {
                    case Z_OK:
                    case Z_DENY:
                    case Z_REJECT:
                    case Z_DROP:
                    case Z_UNSPEC:
                      if(strcmp(self->answer_param,self->answer_p->str))
                        {
                          vissza = FTP_ANS_CHANGE;
                        }
                      else
                        vissza = FTP_ANS_OK;
                      break;
                    case Z_ABORT:
                    default:
                      if(!strcmp(self->answer_param,self->answer_p->str))
                        {
                          g_string_assign(self->answer_c,"500");
                          g_string_assign(self->answer_p,"Error parsing answer, connection dropped.");
                        }
                      vissza = FTP_ANS_ABORT;
                    }
                }
            }
        }
      z_policy_unlock(self->super.thread);
      break;
    case FTP_ANSWER_ABORT:
    default:
      g_string_assign(self->answer_c,"500");
      g_string_assign(self->answer_p,"Error parsing answer, connection dropped.");
      vissza = FTP_ANS_ABORT;
      break;
    }
  return vissza;
}
