/***************************************************************************
 *
 * Copyright (c) 2000 BalaBit IT Ltd, Budapest, Hungary
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: http.h,v 1.20 2000/12/27 14:42:34 bazsi Exp $
 *
 * Author: Balazs Scheidler <bazsi@balabit.hu>
 * Auditor: 
 * Last audited version: 
 * Notes:
 *   Based on the code by: Viktor Peter Kovacs <vps__@freemail.hu>
 *
 ***************************************************************************/

#ifndef ZORP_MODULES_HTTP_H_INCLUDED
#define ZORP_MODULES_HTTP_H_INCLUDED

#include <zorp/zorp.h>
#include <zorp/proxy.h>
#include <zorp/readline.h>

#define HTTP_PASS                 1
#define HTTP_DROP                 2
#define HTTP_POLICY_CALL          3
#define HTTP_CHANGE_NAME          4
#define HTTP_CHANGE_VALUE         5
#define HTTP_CHANGE_BOTH          6
#define HTTP_CHANGE_REGEXP        7
#define HTTP_INSERT               8   /* insert header, don't bother headers with the same name */
#define HTTP_REPLACE              9   /* delete all headers with this name, and add this one */

#define HTTP_CONNECTION_CLOSE		0
#define HTTP_CONNECTION_KEEPALIVE	1

#define HTTP_REQTYPE_SERVER        0     /* simple server request */
#define HTTP_REQTYPE_PROXY         1     /* proxy request */

#define HTTP_REQ_FLG_TRANSFER   1   //supported
#define HTTP_REQ_FLG_STOP       2   //unsupported
#define HTTP_REQ_FLG_DATA       4   //data follows
#define HTTP_REQ_FLG_HEAD       8   //no resp data requested
#define HTTP_REQ_FLG_CLOSE      16  //close connection after transfer
#define HTTP_REQ_FLG_ASTERIX    32  //use '*' instead of '/' if no url
#define HTTP_REQ_FLG_CONNECT    64  //connect as transparent

#define HTTP_RESP_FLG_TRANSFER  1   //supported
#define HTTP_RESP_FLG_STOP      2   //unsupported
#define HTTP_RESP_FLG_CONTINUE  4   //continue (unused)
#define HTTP_RESP_FLG_DATA      8   //data follows
#define HTTP_RESP_FLG_DATA_COND 16  //if not HEAD
#define HTTP_RESP_FLG_CLOSE     32  //close connection after transfer


#define HTTP_MAX_LINE           4096
#define HTTP_MAX_URL            4096
#define HTTP_BLOCKSIZE		4096

#define HTTP_MSG_OK             0
#define HTTP_MSG_CLIENT_SYNTAX  1
#define HTTP_MSG_SERVER_SYNTAX  2
#define HTTP_MSG_POLICY_SYNTAX  3
#define HTTP_MSG_POLICY_VIOLATION 4
#define HTTP_MSG_INVALID_URL    5
#define HTTP_MSG_CONNECT_ERROR  6
#define HTTP_MSG_IO_ERROR	7
#define HTTP_MSG_DEFAULT        8

typedef struct _HttpProxy
{
  ZProxy super;
  ZReadLine *readline[EP_MAX];
  guint timeout;

  GString *headers[EP_MAX];
  guint max_header_lines;
  
  GString *current_header_name, *current_header_value;

  gchar request_method[16];
  guint request_flags;
  GString *request_url;
  gchar request_version[16];
  guint request_type;

  gchar connected_server[256];
  guint connected_port;

  gchar remote_server[256];
  guint remote_port;

  guint default_port;
  
  gchar response_version[16];
  gchar response[4];
  guint response_flags;
  guint response_code;
  gchar response_msg[256];
  
  gboolean connection_mode;        /* 0:close 1:keep-alive */
  gboolean transparent_mode;       /* 0:non-transparent 1:transparent */
  gboolean transparent_server_requests;
  gboolean transparent_proxy_requests;

  GString *parent_proxy;        /* address, or hostname of the parent proxy, if empty direct connection is used */
  guint parent_proxy_port;

  gboolean rewrite_host_header;   /* rewrite host header when redirection is done */

  guint proto_version;          /* version (updated by request and response) : 0x major minor (0x0009 0x0100 0x0101) */

  gint max_line_length;
  gint max_keepalive_requests;

  GHashTable *request_method_policy;     // policy chain to process on request methods
  GHashTable *request_header_policy;     // policy chain to process on request headers
  GHashTable *response_policy;           // policy chain to process on response codes
  GHashTable *response_header_policy;    // policy chain to process on response headers

  int error_code;
  int error_response;
  GString *error_info;
  
#if 0
  int logmode;     //bitmapped: 0b .... .... : err0 err1 err2 err3  evn0 evn1 evn2 evn3
  int maxreqline;  //max. number of lines in the request (command+headers)
  int maxrespline; //max. number of lines in the response (result+headers)
  int maxlinelen;  //max. line length
  int reqflg;      //request command flags
  int respflg;     //response value flags
  int req_cmd_def;                         //default action to take when not found above
  int req_head_def;                        //allowed: PASS | DROP
  int resp_cmd_def;
  int resp_head_def;
  GHashTable* filter_encoding_hash;
  GHashTable* filter_content_hash;
  //filter pointers (they should be filled from the hashes)
  int (*filter_transfer)(struct _HttpProxy* s, int side_src, int side_dst,
			 int (*content_enc)(struct _HttpProxy* s, HttpStr* buf, 
					    int (*content_typ)(struct _HttpProxy* s, HttpStr* buf, int len, int* errorval), 
					    int len, int* errorval),
			 int (*content_typ)(struct _HttpProxy* s, HttpStr* buf, int len, int* errorval),
			 int len, int* errorval);
  int (*filter_encoding)(struct _HttpProxy* s, HttpStr* buf, 
			 int (*content_typ)(struct _HttpProxy* s, HttpStr* buf, int len, int* errorval), 
			 int len, int* errorval);
  int (*filter_content)(struct _HttpProxy* s, HttpStr* buf, int len, int* errorval);
  void* filter_transfer_data;              //save area for filters
  void* filter_encoding_data;
  void* filter_content_data;
  int   (*filter_transfer_error)(void* s); //default error handler for filters
  int   (*filter_encoding_error)(void* s);
  int   (*filter_content_error)(void* s); 
  HttpStr* addr;        //address of the server
  HttpStr* old_addr;    //reconnect if !=addr   => this should contain the name of the connected address 
  int      transp_addr; //connect with transparent chainer (addr ignored)
  int bufcpos;                //client buffer position
  int bufclen;                //client buffer length
  int bufspos;                //server buffer position
  int bufslen;                //server buffer length
  char bufc[HTTP_BLOCKSIZE+1];  //client read buffer
  char bufs[HTTP_BLOCKSIZE+1];  //server read buffer 
#endif
} HttpProxy;

typedef gboolean (*HttpTransferFilter)(HttpProxy *self, gint from, gint to, gint content_length);

typedef guint (*HttpHeaderFilter)(HttpProxy *self, GString *header_name, GString *header_value);

gboolean http_read(HttpProxy *self, int side, gchar *buf, size_t buflen, guint *bytes_read);
gboolean http_write(HttpProxy *self, int side, gchar *buf, size_t buflen);

gboolean http_transfer_filter_copy(HttpProxy *self, int from, int to, gint content_length);
gboolean http_transfer_filter_chunked(HttpProxy *self, int from, int to, gint content_length);

int http_proto_request_lookup(char *req);
int http_proto_response_lookup(char *resp);

gboolean
http_split_url(gchar *url, 
	       gchar **proto, guint *proto_len,
	       gchar **userpw, guint *userpw_len,
	       gchar **host, guint *host_len,
	       guint *port,
	       gchar **file, guint *file_len);

gboolean
http_check_url(gchar *url);

gboolean
http_header_lookup(GString *headers, gchar *what, gchar **result, guint *length, int valueonly);


#endif
