/***************************************************************************
 *
 * Copyright (c) 2000 BalaBit IT Ltd, Budapest, Hungary
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: httpmisc.c,v 1.3 2001/01/11 09:32:48 bazsi Exp $
 *
 * Author: Balazs Scheidler <bazsi@balabit.hu>
 * Auditor: 
 * Last audited version: 
 * Notes:
 *   Based on the code by: Viktor Peter Kovacs <vps__@freemail.hu>
 *
 ***************************************************************************/

#include "http.h"

#include <ctype.h>

gboolean
http_split_url(gchar *url, 
	       gchar **proto, guint *proto_len,
	       gchar **userpw, guint *userpw_len,
	       gchar **host, guint *host_len,
	       guint *port,
	       gchar **file, guint *file_len)
{
  gchar *p, *end;

  *proto = url;
  
  *file = NULL;
  *file_len = 0;
  *port = 0;

  p = url;
  while (*p && *p != ':')
    p++;
    
  if (!*p)
    return FALSE;

  if (*(p + 1) != '/' || *(p + 2) != '/')
    {
      /* protocol not terminated by '//' */
      return FALSE;
    }
  p += 3;
  *proto_len = p - url;

  *host = p;
  while (*p && *p != ':' && *p != '/')
    p++;
  if (*p == ':')
    {
      /* we have a port spec */
      *host_len = p - *host;
      p++;
      *port = strtoul(p, &end, 10);
      if (end == p || (*end != '/' && *end != 0))
	{
	  /* not number, doesn't continue correctly */
	  return FALSE;
	}
      if (*end == 0)
	{
	  /* no filename */
	  return TRUE;
	}
      p = end;
    }
  else if (*p == 0)
    {
      /* no filename part */
      *host_len = strlen(*host);
      return TRUE;
    }
  else if (*p == '/')
    {
      *host_len = p - *host;
    }
  
  *file = p;
  *file_len = strlen(*file);

  return TRUE;
}

gboolean
http_check_url(gchar *url)     //checks correct escapeness
{
  guint length = strlen(url);
  guint t = 0;

  while (t < length)
    {
      switch (url[t])
        {
        case '%':
          if ((t + 2) >= length)
            return FALSE;
            
          if (!isxdigit(url[t + 1]) || !isxdigit(url[t + 2]))
            return FALSE;
          t += 2;
          break;
        default:
          break;
        }
      t++;
    }
  return TRUE;
}

gboolean
http_header_lookup(GString *headers, gchar *what, gchar **result, guint *length, int valueonly)
{
  guint l = strlen(what);
  int t = 0;

  while (headers->str[t] != 0)
    {
      if (strncasecmp(headers->str + t, what, l) == 0)
        {
          char *eol;
          
          if (valueonly)
            {          
              /* skip to value */
              char *colon = strchr(headers->str + t, ':');
              if (colon)
                {
                  colon++;
                  if (*colon++ != ' ')
                    return FALSE;
                  *result = colon;
                  t = colon - headers->str;
                }
              else
		{
		  /* invalid headers */
		  return FALSE;
		}
            }
          else
            {
              *result = headers->str + t;
            }
            
          eol = strchr(headers->str + t, '\n');
          if (eol)
            *length = eol - (headers->str + t) - 1;
          else
            *length = strlen(headers->str + t);
          return TRUE;
        }
      else
        {
          char *next = strchr(&headers->str[t], '\n');
          
          if (!next)
            return FALSE;
          else
            t = next - headers->str;
          t++;
        }
    }
  return FALSE;
}

