############################################################################
##
## Copyright (c) 2000 BalaBit IT Ltd, Budapest, Hungary
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
##
## $Id: Pssl.py,v 1.2.2.1 2001/02/16 10:34:20 bazsi Exp $
##
## Author  : Bazsi
## Auditor : 
## Last audited version: 
## Notes:
##
############################################################################

"""Module defining an interface to the Pssl proxy.

This module defines an interface to the Pssl proxy implemented in Zorp. Pssl
is a plug with the additional possibility of using SSL encryption/decryption
on either of its sides.
"""

from Zorp import *
import Service, Connector, Proxy, Session, Stream

SSL_DEBUG = "ssl.debug"

class PsslProxy(Proxy.Proxy):
	"""Encapsulates an SSL enabled plug proxy.

	This class encapsulates a plug proxy which implements SSL on either
	of its sides.

	Attributes:
	
	  stack_proxy    -- the proxy to stack into Pssl
	  
	  copy_to_client -- Copy data in server->client direction
	  
	  copy_to_server -- Copy data in client->server direction
	  
	  bandwidth_to_client -- Readonly variable containing the utilized
                                 bandwidth in server->client direction.
				 
	  bandwidth_to_server -- Readonly variable containing the utilized
	                         bandwidth in client->server direction.
				 
	  packet_stats_interval -- The number of milliseconds between
	                           two successive packetStats() events.
	                           By default: 0.
	                           NOTE: this is currently implemented as the
	                           number of passing packages, not as
	                           milliseconds.
				   
	  client_need_ssl	-- Use SSL on the client side of the proxy. This
	                     requires setting client_key and client_cert.
				   
	  client_key		-- Client side authentication private key.
	  
	  client_cert		-- Client side authentication certificate (requires a server certificate).
	  
	  server_need_ssl	-- Use SSL on the server side of the proxy. 
	                     Optionally you can set the server_key and 
	                     server_cert attributes if you want to
	                     perform authentication.
				   
	  server_key		-- Server side authentication private key.
	  
	  server_cert		-- Server side authentication certificate (requires a client certificate).
	"""

	def __init__(self, session):
		"""Constructor to initialize a PsslProxy instance.

		Sets attributes based on arguments, and calls the inherited
		constructor.

		Arguments

		  self    -- this instance
		  
		  session -- the reference of the owning session  
		"""
		Proxy.Proxy.__init__(self, "pssl", session)
		self.stack_proxy = None

	def requestStack(self):
		"""Function used to query whether to stack anything to Pssl.

		Callback called by the underlying C proxy to query if
		something is to be stacked.

		Arguments
		
		  self -- this instance
		  
		"""
		if self.stack_proxy != None:
			return self.stackProxy
		return 0

	def stackProxy(self, client_fd, server_fd):
		"""Function to actually stack something into this proxy.
		
		Callback called by the underlying C proxy to actually
		stack in something.

		Arguments
		
		  self -- this instance
		
		  client_fd -- upstream client side fd
		  
		  server_fd -- upstream server side fd
		  
		"""
		Proxy.proxyLog(self, SSL_DEBUG, 7, "stackProxy called: %d, %d" % (client_fd, server_fd))
		subsession = Session.StackedSession(self.session)
		subsession.client_fd = client_fd
		subsession.client_stream = Stream.Stream(client_fd, "pssl client upstream")
		subsession.server_fd = server_fd
		subsession.server_stream = Stream.Stream(server_fd, "pssl server upstream")
		self.stack_proxy(subsession)
