############################################################################
##
## Copyright (c) 2000-2001 BalaBit IT Ltd, Budapest, Hungary
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
##
## $Id: Receiver.py,v 1.11.2.2 2001/02/16 10:33:32 bazsi Exp $
##
## Author  : Bazsi
## Auditor : kisza
## Last version: 1.3
## Notes:
##
############################################################################

"""Module defining the main entry point for Datagram based protocols.

This module defines the class 'Receive', the main entry point for UDP
based protocols.
"""

from Zorp import *
import Zorp, Session, Service

class Receive:
	"""Class encapsulating a datagram receiver.
	
	This class encapsulates a datagram receiver, an interface
	for the ZorpReceiver class implemented in C.
	
	Attributes
	
	  listen	--	the low-level receiver object

	  service	--	the service to start

	  session	--	the session data

	"""

	def __init__(self, bindto, service):
		"""Constructor to initialize a Receive instance.
		
		This constructor initializes a Receive instance by
		setting attributes and creating the underlying
		ZorpReceive instance.
		
		Arguments
		
		  bindto	-- the address to bind to

		  service	-- the service to start for incoming datagrams
		  
		"""
		try:
			self.service = Service.services[service]
		except KeyError:
			raise ServiceException, "Service %s not found" % (service,)

		self.listen = Zorp.Receive(bindto)
		session = Session.MasterSession()
		session.client_fd = self.listen.getFD()
		self.service.startInstance(session)

	def destroy(self):
		"""Function to destroys the receiver
		
		This function can be called to destroy this receiver on
		demand. It calls the destroy method of the low-level
		ZorpReceive object.
		
		Arguments

		  self -- this instance
		"""
		self.listen.destroy()
