############################################################################
##
## Copyright (c) 2000 BalaBit IT Ltd, Budapest, Hungary
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
##
## $Id: policy.py,v 1.49.2.2 2001/02/16 10:34:57 bazsi Exp $
##
############################################################################

from Zorp.Zorp import *
from Zorp import Zorp
from Zorp.Zone import InetZone
from Zorp.Service import Service
from Zorp.SockAddr import SockAddrInet
from Zorp.Chainer import TransparentChainer, DirectedChainer, InbandChainer
from Zorp.Proxy import proxyLog
from Zorp import Http
from Zorp.Http import HttpProxy, HttpProxyNonTransparent
from Zorp.Plug import PlugProxy
from Zorp.Ftp import FtpProxyMinimal
from Zorp.AnyPy import AnyPyProxy
from Zorp.Listener import Listener

from string import split

Zorp.firewall_name = 'bzorp@balabit'

###################
# Zone declaration 
#
InetZone('balabit', '192.168.131.0/24', 
	 outbound_services=["plug"], 
	 inbound_services=["*"])

InetZone('local', '127.0.0.1/8', 
	 outbound_services=["plug", "ftp", "http"], 
	 inbound_services=["*"])

InetZone('internet', '0.0.0.0/0', 
	 inbound_services=["*"])

InetZone('balabit.balabit', ['192.168.131.37', '127.0.0.5'], 
	 outbound_services=["finger"], 
	 admin_parent='balabit')

class MyFinger(AnyPyProxy):
	def proxyThread(self):
		# establish connection
		self.connectServer('', 0)
		client = self.session.client_stream
		server = self.session.server_stream
		user = client.read(128)
		server.write(user)
		response = server.read(132)
		while response:
			client.write(response)
			response = server.read(132)
	
class MyFtp(FtpProxyMinimal):
	def config(self):
		FtpProxyMinimal.config(self)
		self.transparent_mode = 1
		
class MyPlug(PlugProxy):
	def config(self):
		pass #raise NotImplementedError

class MyHttp(HttpProxy):
	def config(self):
                HttpProxy.config(self)
                self.transparent_mode = FALSE
                self.timeout = -1
                self.request_headers["User-Agent"] = \
                        [Http.HTTP_CHANGE_VALUE, "Lynx/2.8.3rel.1 libwww-FM/2.14 SSL-MM/1.4.1 OpenSSL/69.69.4"]
                self.request_headers["Referer"] = \
                        [Http.HTTP_POLICY, self.rereferer]

        def rereferer(self, name, value):
                #self.current_header_value = self.request_url
                return Z_ACCEPT

def zorp():

 	Service("finger", DirectedChainer(SockAddrInet('127.0.01', 79)), MyFinger)
 	Service("plug", DirectedChainer(SockAddrInet('192.168.131.1', 80)), MyPlug)
 	Service("ftp", DirectedChainer(SockAddrInet('192.168.131.1', 21)), MyFtp)
 	Service("http", InbandChainer(), MyHttp)
 	Listener(SockAddrInet("0.0.0.0", 1999), "plug")
	Listener(SockAddrInet('0.0.0.0', 1998), "finger")
	Listener(SockAddrInet('127.0.0.1', 8080), "http")
	Listener(SockAddrInet('127.0.0.1', 2121), "ftp")
